#include "sgl.h"
#include "sgl_cd.h"
#include "z80.h"

#define MAX_FILE 128

void LoadFile(char *name, Uint8 *location, Uint32 size);

unsigned char RAM[0x8000];
Z80_Regs regs;

int Z80_IRQ=-1;
int Z80_Iperiod = 12000;


int DipSwitchSYNC = 0;

int main(int argc,char **argv)
{
slInitSystem(TV_320x480, NULL, 1) ;
slPrint("Generic Arcade Emulator v0.02", slLocate(1,1));
slPrint("ported by Denis aka Dagolar", slLocate(1,2));
slPrint("Thanks goes to TakaIsSilly, Charles Doty", slLocate(1,3));
slPrint("Jim Skilskyj and Marcel de Kogel.", slLocate(1,4));
slPrint("Without them it would be impossible", slLocate(1,5));
slPrint("Controls:", slLocate(1,7));
slPrint("Right shift - insert coin", slLocate(1,8));
slPrint("START       - p1 start", slLocate(1,9));
slPrint("C           - fire", slLocate(1,10));
slPrint("B           - shield", slLocate(1,11));
slPrint("D-pad       - right/left", slLocate(1,12));
slPrint("PRESS START to continue", slLocate(1,14));
while(TRUE) {
        if(Per_Connect1) {
            PerDigital* pad;

            pad = Smpc_Peripheral + 0;
            if(!(pad->data & PER_DGT_ST)) break;
        }
        slSynch();
}
ClsSlo();
if (Load_ROMs() == 0)
        {
        if (run_machine() != 0)
                {
                slPrint("unable to start emulation", slLocate(1,2));
                machine_exit();
                }
        else slPrint("unable to initialize system", slLocate(1,3));
        machine_exit();
        }
exit(0);
}

int run_machine()
{
Z80_Reset();
Z80();
return (0);
}

int Load_ROMs()
{
slPrint("Loading Roms...", slLocate(1,1));
LoadFile("IC45.BIN", &RAM[0x0000], 1);
LoadFile("IC46.BIN", &RAM[0x0800], 1);
LoadFile("IC47.BIN", &RAM[0x1000], 1);
LoadFile("IC48.BIN", &RAM[0x1800], 1);
LoadFile("IC49.BIN", &RAM[0x2000], 1);
LoadFile("IC50.BIN", &RAM[0x2800], 1);
LoadFile("IC51.BIN", &RAM[0x3000], 1);
LoadFile("IC52.BIN", &RAM[0x3800], 1);
return(0);
}

int machine_exit()
{
slPrint("This will exit the machine", slLocate(1,15));
exit(0);
}

unsigned Z80_RDMEM(dword A)
{
Uint16 data;

data = Smpc_Peripheral[0].data;

if (A >= 0x7800 && A <=0x7bff)

        {
        if(DipSwitchSYNC == 1)
                {
                ClsSlo();
                VideoDraw();
                DipSwitchSYNC = 0;
                return 128;
                }
                else
                return 0;
        }
else if (A >= 0x7000 && A <= 0x73FF)
	{
	byte hexdata = 0xff;
	if ((data & PER_DGT_TR) == 0)hexdata &= ~(1<<0);  // Right shift = coin 1
        if ((data & PER_DGT_ST) == 0)hexdata &= ~(1<<1);  // START button = 1p start
	if ((data & PER_DGT_TC) == 0)hexdata &= ~(1<<4); // C button = fire
	if ((data & PER_DGT_KR) == 0)hexdata &= ~(1<<5); // D-Right = right
	if ((data & PER_DGT_KL) == 0)hexdata &= ~(1<<6); // D-Left = left
	if ((data & PER_DGT_TB) == 0)hexdata &= ~(1<<7); // B button = shield
	return hexdata; 
	}
        else if (A < 0x8000)
        return RAM[A];
        slPrint("WARNING! Reading from", slLocate(1,16));
        return RAM[A];
}


void Z80_WRMEM(dword A,byte V)
{
if (A < 0x3000)
	{
	return;
	}
	else if(A > 0x3fff && A < 0x8000)
	RAM[A] = V;
return;
}

int InterruptsEnabled(void)
{
return;
}

byte Z80_In(byte Port)
{
return(0);
}
void Z80_Out(byte Port,byte Value)
{
return;
}
void Z80_Reti(void)
{
return;
}
void Z80_Retn(void)
{
return;
}

int Z80_Interrupt(void)
{
DipSwitchSYNC = 1;
return(Z80_IGNORE_INT);
}

void Z80_Patch(Z80_Regs *Regs)
{ 
slPrint("Blank Patch", slLocate(1,16));
return;
}

void Debug()
{
slPrint("Blank Debugger", slLocate(1,16));
return;
}

int ClsFast(){
int i;
Uint8 *ASCITile;

ASCITile = (Uint8 *)( VDP2_VRAM_A0 );   //need to know propertly location
for(i = 0; i <= 0x60000; i++) *(ASCITile++) = 0x00;
}


int ClsSlo(){
int x2,y2;
for(x2 = 0; x2 <= 40; x2++){
  for(y2 = 0; y2 <= 40; y2++){
    slPrint(" ", slLocate(x2,y2));
    }
  }
}

int VideoDraw(dword A)
{
int x,y;
unsigned char c;
	for(y = 0; y < 32; y++)
		{
		int pos = 0x4000+32*(26-1)+y;
		for (x = 0; x < 26; x++)
			{
			c = RAM[pos];
			if(c>=1 && c<=27)
			c+='A'-1;
			else if (c==31)
			c='*';
			else if (c > 30 && c < 42)
			c+= 16;
			else if (c == 43)
			c='-';
			else if (c == 42)
			c='.';
			else if (c > 0x00)
			c='X';
			slPrint(&c, slLocate(x,y));
			pos -= 32;
	         	}
                }
	for(y = 3; y < 32; y++)
		{
		int pos = 0x4800+32*(26-1)+y;
		for (x = 0; x < 26; x++)
			{
			c = RAM[pos];
			if(c>=1 && c<=27)
			c='.';
			else if (c==31)
			c='*';
			else if (c > 30 && c < 42)
			c= '.';
			else if (c == 43)
			c='-';
			else if (c == 42)
			c='.';
			else if (c > 0x00)
			c='O';
         		slPrint(&c, slLocate(x,y));
			pos -= 32;
			}
        	}
return(0);
}


void LoadFile(char *name, Uint8 *location, Uint32 size)
{

Sint32 dirwork[SLCD_WORK_SIZE(MAX_FILE)];
Sint32 ndir;
CDHN cdhn;
CDKEY key[2];
CDBUF buf[2];
Sint32 stat;
Sint32 len[2];

ndir = slCdInit(MAX_FILE, dirwork);
key[0].cn = key[0].sm = key[0].ci = CDKEY_NONE;
key[1].cn = CDKEY_TERM;
cdhn = slCdOpen(name, key);
buf[0].type = CDBUF_COPY;
buf[0].trans.copy.addr = location;
buf[0].trans.copy.unit = CDBUF_FORM1;
buf[0].trans.copy.size = size;
buf[1].type = CDBUF_TERM;

slCdLoadFile(cdhn, buf);
while (1) {
slSynch();
stat = slCdGetStatus(cdhn, len);
if (stat == CDSTAT_COMPLETED) break;
}
}
